#ifndef FILE_ACCESS_OBJECT_H
#define FILE_ACCESS_OBJECT_H

#include <reflex/base/FileSystem.hpp>


using namespace std;

namespace reflex
{

class FileAccessObject
{
public:

    FileAccessObject();
    FileAccessObject(int type, string tag);
    virtual ~FileAccessObject();

    void setPath(string path);

    virtual int save() const;
    virtual int save(const string& filename);
    virtual int save(cv::FileStorage& node, const string& field) const;
    virtual int load(const string& filename);
    virtual int load(const cv::FileNode& node, const string& path = string(""));
    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    int getError() const { return _err; };
    int getType() const { return _type; };
    string getTag() const { return _tag; };
    string getPath() const { return _path; };
    string getFolder() const { return _folder; };
    std::vector<string> getFolders() const;
    string getFileName() const { return _fileName; };
    string getFileNameBase() const { return _fileNameBase; };
    string getFileNameEnding() const { return _fileNameEnding; };

protected:

    template <class T>
    static inline bool loadField(const cv::FileNode& node, T& value)
    {
        if(node.empty())
            return false;

        node >> value;
        return true;
    }


    template <class T>
    static inline bool loadField(const cv::FileNode& node, std::vector<T>& value)
    {
        if(node.empty())
            return false;

        value.clear();
        for(cv::FileNodeIterator itNode = node.begin() ; itNode != node.end(); ++itNode)
        {
            T val;
            loadField(*itNode, val);
            value.push_back(val);
        }

        return true;
    }


    template <class T>
    static inline void saveField(cv::FileStorage& fs, string key, T& value)
    {
        fs << key << value;
    }


    int _err;
    int _type;
    string _tag;
    string _path;
    string _folder;
    string _fileName;
    string _fileNameBase;
    string _fileNameEnding;
};

} // end namespace reflex


namespace cv
{

    void write(cv::FileStorage& fs, const string&, const reflex::FileAccessObject* obj);
    void read(const FileNode& node, reflex::FileAccessObject* obj, const reflex::FileAccessObject* defaultValue = NULL);
}

#endif
