#ifndef RECTIFICATION_PARAMETERS_H
#define RECTIFICATION_PARAMETERS_H

#include <reflex/base/FileAccessObject.hpp>
#include <reflex/camera/RectificationMap.hpp>


namespace reflex{

class RectificationParameters : public FileAccessObject
{
public:

    RectificationParameters();
    RectificationParameters(const string& fn);

    virtual ~RectificationParameters();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    int loadRectificationMap();
    void computeRectificationMap(const cv::Mat& cameraMatrix, const cv::Mat& distCoeffs);
    int saveRectificationMap();

    cv::Mat R;
    cv::Vec3d rvec;     // rotation vector (rodrigues transform) corresponding to matrix R
    cv::Mat P;
    cv::Mat Q;
    cv::Mat Q32;
    double q03, q13, q23, q32, q33;


    cv::Size imageSize;
    double scale;
    int flags;

    RectificationMap map;
    cv::Rect validRoi;       // ROIs delimiting the area in which valid (defined) pixels are found
    std::string camSerialNumber;
    int calibSerialNumber;

protected:

};


} // end namespace reflex

#endif
