#ifndef DRAWING_H
#define DRAWING_H

#include <reflex/reflex.hpp>
#include <reflex/driver/ReflexDriver.hpp>
#include <reflex/detector/Reflector.hpp>
#include <reflex/geometry/Pose3D.hpp>
#include <reflex/camera/CameraIntrinsics.hpp>


namespace reflex
{

class Drawing
{
public:

    static void drawCross(cv::Mat &img, const cv::Point& point, cv::Scalar color, int thickness = 1, int size = 3, int lineType = cv::LINE_8, int shift = 0);

    static void drawCircles(cv::Mat& img, const std::vector<cv::Point2f>& points, int radius, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);

    static void drawContour(cv::Mat& img, const std::vector<cv::Point2i>& contour, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);
    static void drawContour(cv::Mat& img, const std::vector<cv::Point2f>& contour, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);
    static void drawContour(cv::Mat& img, const std::vector<cv::Point2d>& contour, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);
    static void drawContour(cv::Mat& img, const std::vector<std::vector<cv::Point2i>>& contours, int idx, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);
    static void drawContours(cv::Mat& img, const std::vector<std::vector<cv::Point2i>>& contours, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);
    static void drawContourPoints(cv::Mat& img, const std::vector<std::vector<cv::Point2i>>& contours, int radius, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);

    static void drawReflector(cv::Mat& img, const Reflector& reflector, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);
    static void drawReflectors(cv::Mat& img, std::vector<Reflector>& reflectors, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);

    static void drawCylinder(cv::Mat& img, const CameraIntrinsics& intrinsics, const Pose3f& pose, double length, double radius, int nbPoints, const cv::Scalar& color, int thickness = 1, int lineType = cv::LINE_8);


    static void addWeighted(cv::Mat& img1, const cv::Mat& img2, double alpha, bool excludeBlackAreas = true);

    static cv::Scalar colors[10];

};


} // end namespace reflex

#endif
