#ifndef MAP2D_HH
#define MAP2D_HH

#include <reflex/base/FileAccessObject.hpp>

namespace reflex
{

class Map2D : public FileAccessObject
{
public:

    Map2D();
    virtual ~Map2D();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    void init();
    void init(const cv::Vec2f& borderLower, const cv::Vec2f& borderUpper, double resolution, const cv::Scalar& bgColor);
    void clear();
    void display(const string& title) const;

    cv::Mat getImage() const { return img.clone(); }

    cv::Point2f getPosImage(const cv::Point2f& posMetric) const;
    cv::Point2f getPosImage(const cv::Vec2f& posMetric) const;

    vector<cv::Point2f> getPosImage(const vector<cv::Point2f>& posMetric) const;
    cv::Size2d getSizeImage(const cv::Size2d sizeMetric) const;
    double getSizeImage(double sizeMetric) const;

    void drawFrame(const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA);

    void drawCircle(const cv::Point2f& position, double radius, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void drawCircle(const cv::Vec2f& position, double radius, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);

    void drawLine(const cv::Point2f& point1, const cv::Point2f& point2, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void drawLine(const cv::Vec2f& point1, const cv::Vec2f& point2, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);

    void drawRectangle(const cv::Rect2f& rect, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void drawRectangle(const cv::Point2f& point1, const cv::Point2f& point2, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void drawRectangle(const cv::Point2f& position, const cv::Size2f& size, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void drawRectangle(const cv::Point2f& position, const cv::Size2f& size, double angle, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);

    void drawPolygone(const vector<cv::Point2f>& points, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void drawPolygone(const vector<cv::Vec2f>& points, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);

    void drawPose(const cv::Point2f& position, double angle, double size, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void drawPose(const cv::Vec2f& position, double angle, double size, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);

    void drawGrid(double size, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void drawAxes(const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    void addText(const string& text, const cv::Point2f& position, double size, const cv::Scalar& color, int thickness, int lineType = cv::LINE_AA, float opacity = 1.);
    int  addImage(const cv::Mat& img, const cv::Point& origin, const cv::Mat& mask = cv::Mat());

    cv::Mat img;

    cv::Vec2f borderLower;        // metric lower bound of the visible area
    cv::Vec2f borderUpper;        // metric upper bound of the visible area

    double resolution;            // pixels per meter
    cv::Scalar bgColor;

protected:

    cv::Point2f rotate(const cv::Point2f& point, const cv::Point2f& center, double angle);
    vector<cv::Point2f> rotate(const vector<cv::Point2f>& points, const cv::Point2f& center, double angle);

};


} // end namespace reflex



#endif
