/*
   TCPStream.h

   TCPStream class interface. TCPStream provides methods to trasnfer
   data between peers over a TCP/IP connection.

   ------------------------------------------

   Copyright  2013 [Vic Hargrave - http://vichargrave.com]

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#ifndef __tcpstream_h__
#define __tcpstream_h__

#include "system_defs.h"

#include <sys/types.h>
#if defined(LINUX)
#include <sys/socket.h>
#include <unistd.h>
#endif
#if defined(WINDOWS)
#include <WinSock2.h>
#include <io.h>
#endif

#include <string>
#include <iostream>
#include <errno.h>
#include <string.h>


#if defined(WINDOWS)


#include <BaseTsd.h>
typedef SSIZE_T ssize_t;

#endif

using namespace std;

class TCPStream
{
    int     m_sd;
    string  m_peerIP;
    int     m_peerPort;

  public:
    friend class TCPAcceptor;
    friend class TCPConnector;

    ~TCPStream();

    ssize_t sendit(const char* buffer, size_t len);
    ssize_t full_send(const char *buf, ssize_t len);

    ssize_t receive(char* buffer, size_t len, int timeout=0);
    ssize_t full_receive(char* buffer, ssize_t len);

    string getPeerIP();
    int    getPeerPort();

    enum {
        connectionClosed = 0,
        connectionReset = -1,
        connectionTimedOut = -2
    };

  private:
    bool waitForReadEvent(int timeout);

    TCPStream(int sd, struct sockaddr_in* address);
    TCPStream();
    TCPStream(const TCPStream& stream);
};

#endif
