#ifndef POSE_2D_H
#define POSE_2D_H

#include <reflex/reflex.hpp>
#include <typeinfo>

namespace reflex
{

template<class Tp>
class Pose2
{
public:

    Pose2();
    Pose2(const cv::Mat& T, const cv::Mat& R);
    Pose2(const cv::Vec<Tp,2>& pos, Tp theta);
    virtual ~Pose2();

    cv::Mat operator* (const cv::Mat& pos) const;
    cv::Vec<Tp,2> operator* (const cv::Vec<Tp,2>& pos) const;
    cv::Point_<Tp> operator* (const cv::Point_<Tp>& pos) const;
    Pose2<Tp> operator* (const Pose2<Tp>& pose) const;

    Pose2<Tp> getInverse() const;
    cv::Vec<Tp,2> getPosition() const { return cv::Vec<Tp,2>(T); }
    Tp getAngle() const;

    cv::Mat T;
    cv::Mat R;

    static bool isRotationMatrix(const cv::Mat& R);
};


typedef Pose2<float> Pose2f;
typedef Pose2<double> Pose2d;


} // end namespace reflex


#endif
