#ifndef POSE_3D_H
#define POSE_3D_H

#include <reflex/reflex.hpp>
#include <typeinfo>

namespace reflex
{

template<class Tp>
class Pose3
{
public:

    Pose3();
    Pose3(const cv::Mat& T, const cv::Mat& R);
    Pose3(const cv::Vec<Tp,3>& T, const cv::Vec<Tp,3>& theta);
    Pose3(const cv::Vec<Tp,6>& vec);
    Pose3(const std::vector<Tp>& vec);

    virtual ~Pose3();

    cv::Mat operator* (const cv::Mat& pos) const;
    cv::Vec<Tp,3> operator* (const cv::Vec<Tp,3>& pos) const;
    cv::Point3_<Tp> operator* (const cv::Point3_<Tp>& pos) const;
    Pose3<Tp> operator* (const Pose3<Tp>& pose) const;

    Pose3<Tp> getInverse() const;
    cv::Vec<Tp,3> getPosition() const {return cv::Vec<Tp,3>(T); }
    cv::Vec<Tp,3> getEulerAngles() const { return rotationMatrixToEulerAngles(R); }
    std::vector<Tp> getVector() const;

    int set(const std::vector<Tp>& pose);
    int setPosition(const std::vector<Tp>& position);
    int setAngle(const std::vector<Tp>& angle);

    cv::Mat T;
    cv::Mat R;

    static bool isRotationMatrix(const cv::Mat& R);
    static cv::Vec<Tp,3> rotationMatrixToEulerAngles(const cv::Mat& R);
    static cv::Mat eulerAnglesToRotationMatrix(const cv::Vec<Tp,3>& theta);
};


typedef Pose3<float> Pose3f;
typedef Pose3<double> Pose3d;


} // end namespace reflex


#endif
