#ifndef SENSOR_INFO_H
#define SENSOR_INFO_H

#include <reflex/base/FileAccessObject.hpp>


namespace reflex
{

class SensorInfo : public FileAccessObject
{
public:

    enum types { SENSOR_UNDEFINED = -1, SENSOR_CAMERA = 0, SENSOR_TEMPERATURE };

    SensorInfo(int type = -1);
    virtual ~SensorInfo();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    static cv::Ptr<SensorInfo> create(int type);
    static cv::Ptr<SensorInfo> create(const cv::FileNode& node);
    static cv::Ptr<SensorInfo> create(const string& filename);

};


class CameraInfo : public SensorInfo
{
public:

    enum types { CAMERA_UNDEFINED = -1, CAMERA_MONOCHROME, CAMERA_NIR, CAMERA_RGB };

    CameraInfo();
    virtual ~CameraInfo();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    int cameraType;
    cv::Size resolution;
    cv::Vec2f fov;
};


class TemperatureSensorInfo : public SensorInfo
{
public:

    TemperatureSensorInfo();
    virtual ~TemperatureSensorInfo();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);
};



} // end namespace reflex


#endif
