#ifndef SYSTEM_INFO_H
#define SYSTEM_INFO_H

#include <reflex/base/FileAccessObject.hpp>
#include <reflex/system/SensorInfo.hpp>

namespace reflex
{

class CommChannelInfo : public FileAccessObject
{
public:

    enum types { COMM_UNDEFINED = -1, COMM_ETHERNET };

    CommChannelInfo(int type = -1);
    virtual ~CommChannelInfo();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    static cv::Ptr<CommChannelInfo> create(int type);
    static cv::Ptr<CommChannelInfo> create(const cv::FileNode& node);
    static cv::Ptr<CommChannelInfo> create(const string& filename);
};


class EthernetInfo : public CommChannelInfo
{
public:

    EthernetInfo();
    virtual ~EthernetInfo();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    string ipAddress;
    string port;
};


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


class SystemInfo : public FileAccessObject
{
public:

    SystemInfo(int type = -1);
    virtual ~SystemInfo();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    int getNbSensors() const { return sensors.size(); }
    int getNbCommChannels() const { return commChannels.size(); }

    template<typename T>
    std::vector<cv::Ptr<T>> getSensors(int sensorType) const;

    template<typename T>
    std::vector<cv::Ptr<T>> getCommChannels(int channelType) const;

    int serialNumber;

    std::vector<cv::Ptr<SensorInfo>> sensors;

    std::vector<cv::Ptr<CommChannelInfo>> commChannels;

};


template<typename T>
std::vector<cv::Ptr<T>> SystemInfo::getSensors(int sensorType) const
{
    std::vector<cv::Ptr<T>> result;
    std::vector<cv::Ptr<SensorInfo>>::const_iterator it;
    for(it = sensors.begin(); it != sensors.end(); ++it)
        if((*it)->getType() == sensorType)
            result.push_back(it->staticCast<T>());

    return result;
}


template<typename T>
std::vector<cv::Ptr<T>> SystemInfo::getCommChannels(int channelType) const
{
    std::vector<cv::Ptr<T>> result;
    std::vector<cv::Ptr<CommChannelInfo>>::const_iterator it;
    for(it = commChannels.begin(); it != commChannels.end(); ++it)
        if((*it)->getType() == channelType)
            result.push_back(it->staticCast<T>());

    return result;
}

} // end namespace reflex

#endif
