#ifndef UNIT_INFO_H
#define UNIT_INFO_H

#include <reflex/base/FileAccessObject.hpp>
#include <reflex/system/SystemInfo.hpp>
#include <reflex/geometry/Pose3D.hpp>

namespace reflex
{

class UnitInfo : public FileAccessObject
{
public:

    UnitInfo();

    virtual ~UnitInfo();

    virtual void write(cv::FileStorage& fs) const;
    virtual void read(const cv::FileNode& node);

    static UnitInfo create(int type, const string& serialNumber, const string& ipAddressSys0 = string(""), const string& ipAddressSys1 = string(""));

    int getNbSystems() const { return systems.size(); }
    int getNbSensors() const { return sensors.size(); }

    template<typename T>
    std::vector<cv::Ptr<T>> getSensors(int sensorType) const;

    int unitType;
    std::string serialNumber;

    std::vector<cv::Ptr<SystemInfo>> systems;
    std::vector<cv::Ptr<SensorInfo>> sensors;

    cv::Vec2f fov;
};


template<typename T>
std::vector<cv::Ptr<T>> UnitInfo::getSensors(int sensorType) const
{
    std::vector<cv::Ptr<T>> result;
    std::vector<cv::Ptr<SensorInfo>>::const_iterator it;
    for(it = sensors.begin(); it != sensors.end(); ++it)
        if((*it)->getType() == sensorType)
            result.push_back(it->staticCast<T>());

    return result;
}


} // end namespace reflex


#endif
