#ifndef KALMAN_FILTER_H
#define KALMAN_FILTER_H

#include <reflex/reflex.hpp>
#include <reflex/base/Time.hpp>

namespace reflex
{

class KalmanFilter
{
public:

    class Parameters
    {
    public:
        Parameters();

        int dataType;
        int dimState;
        int dimMeasurement;
        int dimControl;
        cv::Mat transitionMatrix;
        cv::Mat measurementMatrix;
        cv::Mat processNoiseCov;
        cv::Mat measurementNoiseCov;
    };

    KalmanFilter();
    KalmanFilter(const Parameters& parameters, const cv::Mat& state);
    KalmanFilter(int dimState, int dimMeasurement, int dimControl, int dataType, const cv::Mat& state,
                 const cv::Mat& transitionMatrix, const cv::Mat& measurementMatrix,
                 const cv::Mat& processNoiseCov, const cv::Mat& measurementNoiseCov);
    virtual ~KalmanFilter();

    virtual void init(const Parameters& parameters, const cv::Mat& state);
    virtual void init(int dimState, int dimMeasurement, int dimControl, int dataType, const cv::Mat& state,
                      const cv::Mat& transitionMatrix, const cv::Mat& measurementMatrix,
                      const cv::Mat& processNoiseCov, const cv::Mat& measurementNoiseCov);

    virtual cv::Mat predict(const cv::Mat& control = cv::Mat());
    virtual cv::Mat correct(const cv::Mat& measurement);
    virtual cv::Mat getState() const { return kf.statePost; }

    cv::KalmanFilter kf;
};

} // end of namespace reflex


#endif
